/* Pawe Ksik, grupa E1ISIII
ZPR praca domowa
Przykad wykorzystania C++ std::locale
 
Przykad ten przedstawia uycie podstawowych funkcji lokalizacji dostpnych w bibliotece standardowej jzyka C++ 

Uwaga: Usugi lokalizacyjne s zalene od danej implementacji i mog dziaa w rny sposb na rnych platformach.
Przykadowo format daty w lokalizacji polskiej moe by rny na systemach Windows i Linux.
Zakres obsugiwanych lokalizacji jest rwnie zaleny od platformy.
Ten przykad by testowany na serwerze galera.ii.pw.edu.pl (Red Hat Linux 4.1.2-13)*/

#include <iostream>
#include <sstream>
#include <locale> //plik nagwkowy wymagany przez usugi lokalizacyjne
#include <vector>
#include <ctime>
#include <stdexcept>

using namespace std;

typedef vector<locale> LocVector;
typedef vector<locale>::const_iterator LocVectorIter;
typedef istreambuf_iterator<char,char_traits<char> > IstreamIter;
typedef ostreambuf_iterator<char,char_traits<char> > OstreamIter;

int main()
{
	LocVector locVector;

	/*Utworzenie domylnego obiektu lokalizacji - obiekt ten jest skojarzony z ustawieniami
	jzykowymi charakterystycznymi dla systemu operacyjnego
	Tworzymy go podajc pusty cig znakw jako argument konstruktora klasy locale */
	locale defaultLoc("");

	try
	{
		// tworzenie innych obiektw lokalizacji
		locVector.push_back(locale());	//lokalizacja globalna - standardowe US English ASCII
		locVector.push_back(locale("dutch")); //lokalizacja dla jzyka holenderskiego
		locVector.push_back(locale("polish")); //lokalizacja dla jzyka polskiego	
		locVector.push_back(locale("german")); //lokalizacja dla jzyka niemieckiego
		locVector.push_back(locale("swedish")); //lokalizacja dla jzyka szwedzkiego
		
		//Uwaga: Nazwy obiektw lokalizacji (np. polish, german) nie s ustandaryzowane i mog
		//by inne w zalenosi od danej implementacji i systemu operacyjnego.
		//
		//W systemach Linux mona wykorzysta polecenie systemowe locale -a 
		//do sprawdzenia nazw obsugiwanych lokalizacji.
	}
	catch(const std::runtime_error& e)
	{
		//jeeli dana lokalizacja nie jest obsugiwana konstruktor locale() rzuca wyjtek runtime_error
		cout<<e.what()<<endl;
		exit(1);
	}

	/*Obiekty klasy std::locale udostpniaj usugi lokalizacyjne za pomoc obiektw zwanych aspektami (ang. facets).
	Biblioteka standardowa oferuje kilkanacie predefiniowanych aspektw, w tym pliku przedstawi ich
	wykorzystanie na przykadzie aspektw numeric_put, numeric_get, time_put, money_put, money_get oraz collate*/
	
	/*Wykorzystanie aspektu numeric_put - aspekt ten umoliwia wypisywanie do strumienia liczb oraz wartoci
	binarnych w przyjtym przez dan lokalizacj formacie.
	Najwygodniejsz metod korzystania z aspektw numeric_put oraz numeric_get jest wykorzystanie
	operatorw wejcia, wyjcia.
	Pozostae aspekty nie posiadaj tych operatorw co powoduje konieczno posugiwania si specjalnym
	szablonem use_facet<>.*/
	bool bval = true;
	long lval = 644626L;
	unsigned long ulval = 23425334UL;
	double dval = 364657.6274; 
	long double ldval = 127384212.7573;
	cout.precision(15);

	for(LocVectorIter it = locVector.begin(); it!=locVector.end(); ++it)
	{
		//wywoanie metody imbue strumienia powoduje ustawienie go w tryb pracy z okrelona w parametrze lokalizacj
		cout.imbue(*it);
		
		//Metoda name klasy locale pobiera nazw lokalizacji
		cout<<"Lokalizacja: "<<it->name()<<endl;
		cout<<"Warto bool: "<<bval<<endl;
		cout<<"Warto long: "<<lval<<endl;
		cout<<"Warto unsigned long: "<<ulval<<endl;
		cout<<"Warto double: "<<dval<<endl;
		cout<<"Warto long double: "<<ldval<<endl<<endl;	
	}
	cout<<endl;

	/*Wykorzystanie aspektu numeric_get - aspekt ten umoliwia odczyt ze strumienia liczb oraz wartoci
	binarnych w przyjtym przez dan lokalizacj formacie*/
	stringstream stringStream;
	stringStream.precision(15);
	cout.imbue(defaultLoc);
	stringStream.imbue(defaultLoc);
	stringStream<<ldval;	//zapisujemy warto long double do strumienia (aspekt numeric_put)
	ldval = 0;
	stringStream>>ldval;	//odczytujemy warto long double ze strumienia (aspekt numeric_get)
	cout<<"numeric_get odczyta: "<<ldval<<endl<<endl;
	
	/*Przykad uycia aspektu time_put - aspekt ten umoliwia wypisywanie do strumienia daty i czasu
	w przyjtym przez dan lokalizacj formacie*/
	time_t tt = time(NULL);
	struct tm* tm_buf = localtime(&tt);
	struct tm currentTime;
	memcpy(&currentTime,tm_buf,sizeof(struct tm));
	//struktura currentTime zawiera teraz aktualny czas lokalny

	for(LocVectorIter it = locVector.begin(); it!=locVector.end(); ++it)
	{
		cout.imbue(*it);
		cout<<"Lokalizacja "<<it->name()<<", data i czas: ";

		//uywamy szablonu use_facet<> w celu skorzystania z aspektu time_put
		//parametry metody put aspektu time_put:
		//	- iterator wyjciowy strumienia
		//	- strumie wyjciowy
		//	- znak wypenienia uywany przy wyrwnywaniu wartoci w polu o wikszej szerokoci
		//	- wskanik na struktur tm
		//	- tryb wypisywania 'c' oznacza wypisanie penej daty i czasu
		use_facet<time_put<char,OstreamIter> >(*it).put(OstreamIter(cout),cout,cout.fill(),&currentTime,'c');
		cout<<endl;
	}
	/*
	W bibliotece standardowej zaimplementowano rwnie aspekt time_get sucy do odczytu ze strumienia
	czasu zapisanego w formacie danej lokalizacji, jednake w dostpnych dla mnie implementacjach C++
	nie udao mi si go zastosowa w sposb dajcy waciwy rezultat.
	*/
	cout<<endl;
	/*
	Przykad wykorzystania aspektu money_put - jest to formatowane wyjcie dla wartoci pienienych
	*/
	long double moneyval = 231623; 	//warto pienina podawana jest w najmniejszej dla danej waluty jednostkach
					//np dla waluty polskiej warto ta oznacza 2316.23 z
					
	for(LocVectorIter it = locVector.begin(); it!=locVector.end(); ++it)
	{
		stringStream.clear();
		stringStream.str("");
		stringStream.imbue(*it);
		cout<<"Lokalizacja: "<<it->name()<<", format wartoci pieninej: ";
		
		//parametry metody put aspektu money_put:
		//	- iterator wyjciowy strumienia
		//	- warto boolean okrelajca czy wypisywa symbol waluty w formie lokalnej (false) czy midzynarodowej (true)
		//	- strumie wyjciowy
		//	- znak wypenienia uywany przy wyrwnywaniu wartoci w polu o wikszej szerokoci
		//	- warto pienina
		use_facet<money_put<char,OstreamIter> >(*it).put(OstreamIter(stringStream),false,stringStream,stringStream.fill(),moneyval);
		cout<<stringStream.str()<<endl;
	}
	cout<<endl;
	/*
	Przykad wykorzystania aspektu money_get - analizuje strumie w celu odczytania wartoci pieninej w formacie danej lokalizacji
	*/			
	ios_base::iostate state;
	cout.imbue(locVector.back());
	//cout<<"Lokalizacja: "<<it->name()<<", format wartoci pieninej: ";
		
	moneyval=0;
	//parametry metody get aspektu money_get:
	//	- iterator wejciowy strumienia wskazujcy miejsce od ktrego nalezy rozpocz wczytywanie
	//	- iterator wejciowy strumienia wskazujcy miejsce przed ktrym nalezy zakoczy wczytywanie
	//	- warto boolean okrelajca czy stosowany jest symbol waluty w formie lokalnej(false) czy midzynarodowej (true)
	//	- strumie wejciowy
	//	- referencja na stan strumienia - gdy odczyt wartoci zakoczy si niepowodzeniem ustawiany jest bit ios_base::failbit w zmiennej state
	//	- referencja na warto pienin
	use_facet<money_get<char,IstreamIter> >(locVector.back()).get(IstreamIter(stringStream),IstreamIter(),false,stringStream,state,moneyval);
	cout<<"money_get odczyta "<<moneyval<<" z "<<stringStream.str()<<endl<<endl;

	
	string str1("ala");
	string str2("kot");
	cout.imbue(defaultLoc);

	/*Przykad uycia aspektu collate - aspekt ten umoliwia porwnywanie alfabetyczne sw wedug przyjtej
	w danej lokalizacji kolejnoci liter alfabetu oraz udostpnia funkcj mieszajc.
	
	Metoda compare aspektu collate porwnuje alfabetycznie (zgodnie z konwencj w danej lokalizacji)
	dwa cigi znakw i zwraca:
	-1 - gdy pierwsze sowo poprzedza alfabetycznie drugie
	0 - gdy obie sekwencje s rwne
	-1 - gdy drugie sowo poprzedza alfabetycznie pierwsze*/
	cout<<"compare: "<< use_facet<collate<char> >(defaultLoc).compare(str1.c_str(), str1.c_str()+str1.length(),
				str2.c_str(), str2.c_str()+str2.length()) <<endl;

	/*
	Metoda hash aspektu collate oblicza warto funkcji mieszajcej dla danego cigu znakw
	*/
	cout<<"hash: "<< use_facet<collate<char> >(defaultLoc).hash(str1.c_str(), str1.c_str()+str1.length()) <<endl;
	
	return 0;
}
